// --- [ main.c ] -------------------------------------------------------------
//
// tab = 3
//
// Test zum Aufwecken aus dem Sleep-Modus
// fr ATtiny 2313 mit XTAL 3.686.400
//
// rx (PD0) muss mit PCINT1 (PB1) verbunden werden.
// der Timer0 gibt ein 10kHz Signal an PB2 aus (nicht whrend des Sleep)
// PB0 ist high, wenn der Controller aktiv ist.
//
// ----------------------------------------------------------------------------

#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <stdlib.h>
#include "global.h"
#include <util/delay.h>
#include "rs232_read.h"


int main(void)
{
    uint8_t i, cnt;
    uint8_t buffer[16];
    RS232_init();
    sei();

    // damit die Abfrage "while (~(UCSRA & (1<<TXC)));" funktioniert
    // TXC Transmit Complete
    put_c(0x00);

    // Pins
    DDRB   = (1<<PB0 | 1<<PB2);
    // Pullup an PCINT1 (PB1) setzen
    PORTB |= (1<<PB0 | 1<<PB1 | 1<<PB2);
    // Pullup an INT0 setzen
    PORTD |= (1<<PD2);
    // Timer0
    TCCR0A = (0<<COM0A1 | 1<<COM0A0 | 1<<WGM01 | 0<<WGM00); // Toogle at Compare Match
    TCCR0B = (0<<WGM02 | 1<<CS00);   // Prescale 1
    OCR0A = 184;
    TCNT0 = 0;

    while (TRUE)
    {
        // warten, bis eine laufende Sendeaktion abgeschlossen ist
        // dazu muss aber mindestens ein Byte abgeschickt worden sein,
        // sonst gibt hier eine Endlosschleife.
        while ((UCSRA & (1<<TXC)) == 0);
        UCSRA |= (1<<TXC);

        // Interrupt setzen
        // INT0 als low-level
        // MCUCR = (0<<ISC01 | 0<<ISC00);
        PCMSK = (1<<PCINT1);
        GIMSK = (1<<PCIE);

        RS232_stop();
        RS232_Clear_Counter();

        // sleep
        MCUCR = (1<<SE | 1<<SM1 | 1<<SM0);      // Powerdown
        // PB0 auf low, PB0 ist whrend des sleep low
        PORTB &= (~(1<<PB0));

        asm volatile("sleep");

        // Interrupt ausgelst (durch PCINT1)
        // INT deaktivieren
        GIMSK = 0;
        // MCUCR = 0;

        // PB0 auf high
        PORTB |= (1<<PB0);

        // warten, bis der INT0-PIN auf high geht (am Ende von 0xFF)
        // while ((PIND & (1<<INT0_PIN)) == 0);
        // rs232 aktivieren
        RS232_start();

        // Ein Lebenszeichen schicken
        put_c(0x5a);
        // continue;

        // und nun bis zu ca. 25ms auf eingehende Daten warten
        cnt = 255;
        while (cnt--)
        {
            _delay_us(110);
            i = RS232_Count_Bytes();
            // empfangene Daten zurcksenden
            if (i)
            {
                RS232_Get_Bytes(buffer);
                RS232_Send_Bytes(buffer, i);
                // das nchste Byte sollte nun schneller eintreffen
                cnt = 10;
            }
        }
    }
}

EMPTY_INTERRUPT(INT0_vect);
EMPTY_INTERRUPT(PCINT_vect);

// --- [ eof ] ----------------------------------------------------------------
