// --- [ rs232_read.c ] -------------------------------------------------------

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include "global.h"

#define RX_BUFFER_SIZE	32
#define TX_BUFFER_SIZE	32
#define READ_BYTES      1

#define BAUD  	9600
#define TEILER ((F_CPU /(BAUD * 16l)) -1)

// Register USCRB konfigurieren (ATTINY2313)
#define ENABLE_RX() 	(UCSRB = (1<<RXCIE | 0<<TXCIE | 0<<UDRIE | 1<<RXEN | 1<<TXEN))
#define DISABLE_RX()	(UCSRB = (0<<RXCIE | 0<<TXCIE | 0<<UDRIE | 0<<RXEN | 1<<TXEN))

// --- lokale Variablen -------------------------------------------------------

uint8_t rx_buffer[RX_BUFFER_SIZE];
volatile uint8_t rx_head;
volatile uint8_t rx_flag;

// ----------------------------------------------------------------------------
// initialisiert die RS232-Schnittstelle
void RS232_init(void)
{
    // ATTINY2313
    UBRRL = TEILER;
    UBRRH = 0;
    UCSRC = (0<<UMSEL) | (1<<UCSZ1) | (1<<UCSZ0);
    rx_flag = FALSE;
    rx_head = 0;
    ENABLE_RX();
}

// -----------------------------------------------------------------------------
void RS232_stop()
{
    DISABLE_RX();
}

// -----------------------------------------------------------------------------
void RS232_start(void)
{
    ENABLE_RX();
}

// -----------------------------------------------------------------------------
uint8_t RS232_Count_Bytes(void)
// zeigt an, ob bzw. wieviele Bytes empfangen wurden
{
    return(rx_head);
}

// ATTINY2313
// ----------------------------------------------------------------------------
// Es werden 4 Byte empfangen, danach wird das Flag rx_flag gesetzt.

ISR (USART_RX_vect)
{
	uint8_t i;
	i = UDR;
											// den Index incrementieren,
											// sofern der Puffer noch nicht voll
	if (rx_head < RX_BUFFER_SIZE)
	{
		rx_buffer[rx_head++] = i;			// die Daten speichern
	}

	if (rx_head >= READ_BYTES)			// wenn die Daten komplett
	{
        rx_flag = TRUE;						// das Flag setzen
	}
}

// ---------------------------------------------------------------------------
// liefert die empfangenen Bytes via Array zurck und lscht das Flag rx_flag
// damit diese Daten nicht noch ein weiteres Mal ausgelesen werden.
// Die Anzahl der Bytes steht in rx_head

void RS232_Get_Bytes(uint8_t * data)
{
    uint8_t i = 0;
    do	{
        data[i] = rx_buffer[i];
        } while ((i++) < rx_head);

    rx_head = 0;
}

// ---------------------------------------------------------------------------
// Wenn nicht die erwartete Anzahl von Bytes empfangen wurden, dann den Counter
// zurckstellen
void RS232_Clear_Counter(void)
{
    rx_head = 0;
}

// --- Standard-RS232-Routinen --- ATTINY2313 ---------------------------------
// ----------------------------------------------------------------------------
// ein einzelnes Zeichen auf COM ausgeben
void put_c(uint8_t Zeichen)
{
    while(!(UCSRA & (1<<UDRE)));
    UCSRA |= (1<<TXC);	// Flag TXC lschen fr die Auswertung vor dem Sleep
    UDR = Zeichen;
}

void RS232_Send_Bytes(uint8_t * buffer, uint8_t cnt)
{
    uint8_t i = 0;
    while(cnt--) put_c(buffer[i++]);
}

/*
// ----------------------------------------------------------------------------
// einen String auf COM ausgeben
void put_s(char *s)
{
while(*s)
	{
	put_c(*s);
	s++;
	}
}
*/
// --- [ eof ] ----------------------------------------------------------------

